#include <type_traits>

template <typename...>
struct list;

// <front>
template <typename>
struct front_impl;

template <template <typename...> typename L, typename T, typename... Ts>
struct front_impl<L<T, Ts...>> {
    using type = T;
};

template <typename L>
using front = typename front_impl<L>::type;

static_assert(std::is_same_v<front<list<int>>, int>);
static_assert(std::is_same_v<front<list<char, double>>, char>);
static_assert(std::is_same_v<front<list<double, int, char>>, double>);
// </front>

// <first>
template <typename... Ts>
using first = front<list<Ts...>>;

static_assert(std::is_same_v<first<int>, int>);
static_assert(std::is_same_v<first<char, double>, char>);
static_assert(std::is_same_v<first<double, int, char>, double>);
// </first>

// <last>
template <typename T>
struct identity {
    using type = T;
};

template <typename... Ts>
using last = typename decltype((..., identity<Ts>{}))::type;

static_assert(std::is_same_v<last<int>, int>);
static_assert(std::is_same_v<last<char, double>, double>);
static_assert(std::is_same_v<last<double, int, char>, char>);
// </last>

// <back>
template <template <typename...> typename, typename>
struct apply_impl;

template <
    template <typename...> typename F,
    template <typename...> typename L,
    typename... Ts
>
struct apply_impl<F, L<Ts...>> {
    using type = F<Ts...>;
};

template <template <typename...> typename F, typename L>
using apply = typename apply_impl<F, L>::type;

template <typename L> using back = apply<last, L>;

static_assert(std::is_same_v<back<list<int>>, int>);
static_assert(std::is_same_v<back<list<char, double>>, double>);
static_assert(std::is_same_v<back<list<double, int, char>>, char>);
// </back>

int main(int const, char const* const []) {
    return 0;
}